/*
 * VGA display-related functions.
 *
 * Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __VGA_DISPLAY_H__
#define __VGA_DISPLAY_H__

#include "pic32_c.h"
#include "display.h"



/* Display configuration type. */

typedef struct
{
    /* Current display state handler. */

    void (*state_handler)();

    /* DMA transfer properties. */

    int line_channels, initiating_int_num;

    /* Horizontal and vertical sync peripherals. */

    int timer, hsync_unit, vsync_unit;

    /* Current scanline. */

    uint32_t line;

    /* Pointers to pixel lines. */

    uint8_t *linedata;

    /* General display configuration. */

    display_config_t *display_config;

} vga_display_t;



/* Initialisation. */

void init_vga(display_config_t *display_config, int line_channels,
              int initiating_int_num);

void vga_configure_sync(int hsync_unit, int vsync_unit, int timer);

void vga_configure_transfer(int transfer_int_num, uint32_t output);

void vga_configure_line_channel(int channel, int int_num, enum dma_chain chain,
                                uint32_t output);

void vga_configure_zero_channel(int channel, int int_num, int initiating,
                                uint32_t output);

/* Interrupt handlers. */

void vga_interrupt_handler(void);

/* Display state handlers. */

void vbp_active(void);
void visible_active(void);
void vfp_active(void);
void vsync_active(void);

/* Display operations. */

void start_visible(void);
void update_visible(void);
void stop_visible(void);
void update_transfers(int enable);

/* Vertical sync operations. */

void vsync_high(void);
void vsync_low(void);

#endif /* __VGA_DISPLAY_H__ */
